﻿////////////////////////////////////////////////////////////////////////////////
//
// AudioscrobblerReader
//   XMLControler.as
//   Rev. 1.1 (2005/9/25)
// 
// by nomura masayuki
// http://www.a3size.com/AudioscrobblerReader/
//
////////////////////////////////////////////////////////////////////////////////
//
// XMLコントロールクラス
// XMLモデルのコントローラ
//
////////////////////////////////////////////////////////////////////////////////
import com.a3size.audioscrobblerReader.*;

class com.a3size.audioscrobblerReader.XMLControler
{
	// recenttracks.xmlが置かれているURI
	static private var DEFAULT_XMLURI_PREFIX:String = "http://ws.audioscrobbler.com/1.0/user/";
	static private var DEFAULT_XMLURI_SUFFIX:String = "/recenttracks.xml";
	// ユーザーの情報が置かれているURI
	static private var DEFAULT_USERURI_PREFIX:String = "http://www.last.fm/user/";

	// オフラインとみなす長さ(秒)
	static private var OFFLINE_SECONDS:Number = 1800;

	// リロードする間隔(秒)
	static private var RELOAD_SECONDS:Number = 600;
	
	// ロード中表示
	static private var MESSAGE_LOAD_SONG:String = "          .........................     now loading     .........................          ";
	static private var MESSAGE_LOAD_ARTIST:String = "please wait.";

	// XMLが見つからない場合
	static private var MESSAGE_NG_SONG:String = "(not found.)";
	static private var MESSAGE_NG_ARTIST :String= "orz";

	// XMLからアイテムが見つからない場合(1曲も登録がない場合)
	static private var MESSAGE_NO_SONG:String = "(no data.)";
	static private var MESSAGE_NO_ARTIST:String = "-";

	private var item_loading:SongItem;
	private var item_NG:SongItem;
	private var item_noData:SongItem;
	private var songTextField:ScrollTextField;
	private var artistTextField:ScrollTextField;
	private var onlineTogglePanel:TogglePanel;
	private var sharedObj:SharedObject;
	private var xmlFileName:String;
	private var xmlUserName:String;
	private var songXML:RecenttracksXML;
	private var reloadID:Number;
	private var reloadInterval:Number;

	// コンストラクタ
	function XMLControler()
	{
		 songXML = new RecenttracksXML(this);
		 songXML.ignoreWhite = false;
		 
		 item_loading = new SongItem;
		 item_loading.song = MESSAGE_LOAD_SONG;
		 item_loading.artist = MESSAGE_LOAD_ARTIST;
		 item_loading.date = new Date();

		 item_NG = new SongItem;
		 item_NG.song = MESSAGE_NG_SONG;
		 item_NG.artist = MESSAGE_NG_ARTIST;
		 item_NG.date = new Date();

		 item_noData = new SongItem;
		 item_noData.song = MESSAGE_NO_SONG;
		 item_noData.artist = MESSAGE_NO_ARTIST;
		 item_noData.date = new Date();

		 xmlFileName = "";
		 xmlUserName = "";
	}
	
	// コントロールの開始(XMLの取得)
	function startReader():Void
	{
		// XMLファイル名の設定(ユーザー名で指定されている場合)
		if (xmlFileName.length == 0)
			if (xmlUserName.length > 0) xmlFileName = DEFAULT_XMLURI_PREFIX + escape(xmlUserName) + DEFAULT_XMLURI_SUFFIX;

		// XMLファイル名が設定されている場合
		if (xmlFileName.length > 0)
		{
			// グローバル変数にlinkURIを登録
			_global.linkURI = DEFAULT_USERURI_PREFIX + escape(xmlUserName);

			// SharedObject内に最近の読み込み結果がないか
			var sharedFileObj:Object = this.sharedObj.data.XMLfileName;
			var sharedDateObj:Object = this.sharedObj.data.XMLDATE;
			var sharedItemObj:Object = this.sharedObj.data.currentDate;
			if ((sharedFileObj != undefined) && (sharedDateObj != undefined) && (sharedItemObj != undefined))
			{
				var sharedFileName:String = this.sharedObj.data.XMLfileName;
				if (sharedFileName == xmlFileName) // 同じファイルを扱うならば
				{
					var sharedDate:Date = this.sharedObj.data.XMLDATE;
					trace("sharedDate:" + sharedDate.toString());

					// リロード時間が経過していない場合はSharedObjectから復旧させる
					var date:Date = new Date();
					trace("now Date:" + date.toString());
					var elapsedTime:Number = Math.abs(getDiffTime(date, sharedDate));
					if (elapsedTime < (RELOAD_SECONDS * 1000))
					{
						trace(getDiffTime(date, sharedDate) + "  from SharedObject.");
						songXML.xmlDate = sharedDate;
						
						// SharedObjectから曲アイテムを復旧させる
						var sharedItem:SongItem = new SongItem;
						sharedItem.date = this.sharedObj.data.currentDate;
						sharedItem.song = this.sharedObj.data.currentSong;
						sharedItem.artist = this.sharedObj.data.currentArtist;
						setItem(sharedItem);

						// リロードの設定
						setReload((RELOAD_SECONDS * 1000) - elapsedTime);
						return;
					}
				}
			}
		}
		// XML読み込み
		loadXML();
	}

	// XMLの読み込み
	private function loadXML(intervalTime:Number):Void
	{
		stopReload();

		if (isNaN(intervalTime) || intervalTime < 1000)
			intervalTime = RELOAD_SECONDS * 1000;
		if (xmlFileName.length > 0) // XMLのURIが定義されているか
		{
			setItem(item_loading);

			// 読み込み用ダミー文字列の作成
			var dummy_str:String = "";
			if (xmlFileName.indexOf(DEFAULT_XMLURI_PREFIX) == 0) dummy_str = "?" + new Date().getTime();

			// 読み込み
			songXML.resetXML();
			trace("load:" + xmlFileName + dummy_str);
			songXML.load(xmlFileName + dummy_str);
		}
		else
		{
			setItem(item_NG);
		}
		
		trace("next reload interval:" + intervalTime);
		reloadInterval = intervalTime;
	}

	// ロード完了処理
	function loaded(success:Boolean):Void
	{
		if (success)
		{
			var currentSong:SongItem;
			if (songXML.item.length > 0)
				currentSong = songXML.item[0];
			else
				currentSong = item_noData;
			setItem(currentSong);

			// SharedObjectに記録する
			this.sharedObj.data.XMLfileName = xmlFileName;
			this.sharedObj.data.XMLDATE = new Date();
			this.sharedObj.data.currentDate = currentSong.date;
			this.sharedObj.data.currentSong = currentSong.song;
			this.sharedObj.data.currentArtist = currentSong.artist;
			this.sharedObj.flush();
		}
		else
		{
			setItem(item_NG);
		}
		setReload(reloadInterval);
	}
	
	private function setReload(intervalTime:Number):Void
	{
		trace("set reload interval:" + intervalTime);
		this.reloadID = setInterval(this, "loadXML", intervalTime, RELOAD_SECONDS * 1000);
	}
	
	function stopReload():Void
	{
		if (this.reloadID != undefined) clearInterval(this.reloadID);
	}
	
	function setItem(songItem:SongItem):Void
	{
		// 現在時刻との差を判別
		var date:Date = new Date();
		if (Math.abs(getDiffTime(date, songItem.date)) > (OFFLINE_SECONDS * 1000))
		{
			// オフライン
			this.onlineTogglePanel.status = false;
		}
		else
		{
			if ((songItem != item_NG) && (songItem != item_noData) && (songItem != item_loading))
				this.onlineTogglePanel.status = true;
		}
		songTextField.text = songItem.song;
		artistTextField.text = songItem.artist;
	}

	// ファイル名
	function get xmlName():String
	{
		return this.xmlFileName;
	}
	function set xmlName(fileName:String):Void
	{
		this.xmlFileName = fileName;
		if ((this.xmlFileName == undefined) || (this.xmlFileName == null)) this.xmlFileName = "";
	}
	
	// ファイル名
	function get userName():String
	{
		return this.xmlUserName;
	}
	function set userName(str:String):Void
	{
		this.xmlUserName = str;
		if ((this.xmlUserName == undefined) || (this.xmlUserName == null)) this.xmlUserName = "";
	}
	
	// テキストフィールド群
	function get songField():ScrollTextField
	{
		return this.songTextField;
	}
	function set songField(field:ScrollTextField):Void
	{
		this.songTextField = field;
	}
	function get artistField():ScrollTextField
	{
		return this.artistTextField;
	}
	function set artistField(field:ScrollTextField):Void
	{
		this.artistTextField = field;
	}
	
	// オンライン表示トグル
	function set onlineToggle(togglePanel:TogglePanel):Void
	{
		this.onlineTogglePanel = togglePanel;
	}

	function get sharedObject():SharedObject
	{
		return this.sharedObj;
	}
	function set sharedObject(obj:SharedObject):Void
	{
		this.sharedObj = obj;
	}
	
	// 時刻の差をミリ秒で返す
	private function getDiffTime(date_1:Date, date_2:Date):Number
	{
		var time_1:Number = date_1.getTime();
		var time_2:Number = date_2.getTime();
		return (time_1 - time_2);
	}
}